@@ -36,30 +36,29 @@ if advised of the possibility of such damage.
 
 package rice.p2p.multiring.testing;
 
-import rice.*;
+import java.io.IOException;
+import java.net.*;
+import java.util.*;
 
+import rice.environment.Environment;
+import rice.environment.params.simple.SimpleParameters;
+import rice.environment.random.RandomSource;
 import rice.p2p.commonapi.*;
-import rice.p2p.multiring.*;
-
-import rice.pastry.PastryNodeFactory;
-import rice.pastry.NodeId;
-import rice.pastry.NodeIdFactory;
-import rice.pastry.PastryNode;
-import rice.pastry.commonapi.*;
+import rice.p2p.commonapi.Id;
+import rice.p2p.commonapi.NodeHandle;
+import rice.p2p.multiring.MultiringNode;
+import rice.pastry.*;
+import rice.pastry.commonapi.PastryIdFactory;
 import rice.pastry.direct.*;
 import rice.pastry.dist.*;
-import rice.pastry.standard.*;
-
-import java.util.*;
-import java.net.*;
-import java.io.Serializable;
+import rice.pastry.standard.RandomNodeIdFactory;
 
 /**
  * Provides regression testing setup for applications written on top of the
  * commonapi. Currently is written to use Pastry nodes, but this will be
  * abstracted away.
  *
- * @version $Id: MultiringRegrTest.java,v 1.5 2005/03/15 21:12:33 jeffh Exp $
+ * @version $Id: MultiringRegrTest.java 2896 2006-01-09 17:11:50Z jeffh $
  * @author Alan Mislove
  */
 public class MultiringRegrTest {
@@ -102,12 +101,6 @@ public class MultiringRegrTest {
    */
   protected Id[] ringIds;
 
-  // the random id generatoe
-  /**
-   * DESCRIBE THE FIELD
-   */
-  protected Random rng;
-
   // ----- PASTRY SPECIFIC VARIABLES -----
 
   // the factory for creating pastry nodes
@@ -128,6 +121,11 @@ public class MultiringRegrTest {
    */
   protected NetworkSimulator simulator;
 
+  /**
+   * DESCRIBE THE FIELD
+   */
+  public Environment environment;
+
   // ----- STATIC FIELDS -----
 
   // the number of nodes to create
@@ -160,12 +158,6 @@ public class MultiringRegrTest {
    */
   public static int NUM_ORGANIZATIONAL_NODES = NUM_GATEWAY_NODES + NUM_INTERNAL_NODES;
 
-  // the factory which creates pastry ids
-  /**
-   * DESCRIBE THE FIELD
-   */
-  public static IdFactory FACTORY = new PastryIdFactory();
-
   // ----- TESTING SPECIFIC FIELDS -----
 
   // the text to print to the screen
@@ -238,33 +230,36 @@ public class MultiringRegrTest {
    */
   public static String INSTANCE_NAME = "MultiringRegrTest";
 
-
   // ----- EXTERNALLY AVAILABLE METHODS -----
 
   /**
    * Constructor, which takes no arguments and sets up the factories in
    * preparation for node creation.
+   *
+   * @param env DESCRIBE THE PARAMETER
+   * @exception IOException DESCRIBE THE EXCEPTION
    */
-  public MultiringRegrTest() {
+  public MultiringRegrTest(Environment env) throws IOException {
+    this.environment = env;
     if (SIMULATOR == SIMULATOR_SPHERE) {
-      simulator = new SphereNetwork();
+      simulator = new SphereNetwork(env);
     } else {
-      simulator = new EuclideanNetwork();
+      simulator = new EuclideanNetwork(env);
     }
 
     if (PROTOCOL == PROTOCOL_DIRECT) {
-      factory = new DirectPastryNodeFactory(new RandomNodeIdFactory(), simulator);
+      factory = new DirectPastryNodeFactory(new RandomNodeIdFactory(environment), simulator, env);
     } else {
-      factory = DistPastryNodeFactory.getFactory(new RandomNodeIdFactory(),
+      factory = DistPastryNodeFactory.getFactory(new RandomNodeIdFactory(environment),
         PROTOCOL,
-        PORT);
+        PORT,
+        env);
     }
 
     NUM_GATEWAY_NODES = NUM_GLOBAL_NODES / NUM_ORGANIZATIONS;
     NUM_ORGANIZATIONAL_NODES = NUM_GATEWAY_NODES + NUM_INTERNAL_NODES;
 
-    rng = new Random();
-    idFactory = new PastryIdFactory();
+    idFactory = new PastryIdFactory(env);
     globalRingId = idFactory.buildId(new byte[20]);
     ringIds = new Id[NUM_ORGANIZATIONS];
     globalNodes = new MultiringNode[NUM_GLOBAL_NODES];
@@ -421,7 +416,7 @@ public class MultiringRegrTest {
    */
   private Id generateId() {
     byte[] data = new byte[20];
-    rng.nextBytes(data);
+    environment.getRandomSource().nextBytes(data);
     return idFactory.buildId(data);
   }
 
@@ -446,6 +441,7 @@ public class MultiringRegrTest {
    * have been created and are ready.
    */
   protected void runTest() {
+    RandomSource rng = environment.getRandomSource();
     for (int i = 0; i < 20; i++) {
       int si = rng.nextInt(NUM_ORGANIZATIONS);
       int sj = rng.nextInt(NUM_ORGANIZATIONAL_NODES);
@@ -706,17 +702,19 @@ public class MultiringRegrTest {
    * The main program for the MultiringRegrTest class
    *
    * @param args The command line arguments
+   * @exception IOException DESCRIBE THE EXCEPTION
    */
-  public static void main(String[] args) {
+  public static void main(String[] args) throws IOException {
     parseArgs(args);
-    MultiringRegrTest test = new MultiringRegrTest();
+    MultiringRegrTest test = new MultiringRegrTest(new Environment(null, null, null, new DirectTimeSource(System.currentTimeMillis()), null,
+      new SimpleParameters(Environment.defaultParamFileArray, null)));
     test.start();
   }
 
   /**
    * DESCRIBE THE CLASS
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   public class MultiringTestApp implements Application {
@@ -777,7 +775,7 @@ public class MultiringRegrTest {
   /**
    * DESCRIBE THE CLASS
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   public static class MultiringTestMessage implements Message {
