@@ -42,6 +42,9 @@ import java.util.logging.*;
 
 import rice.*;
 import rice.Continuation.*;
+import rice.environment.Environment;
+import rice.environment.logging.Logger;
+import rice.environment.params.Parameters;
 import rice.p2p.commonapi.*;
 import rice.p2p.past.PastPolicy.*;
 import rice.p2p.past.messaging.*;
@@ -52,17 +55,32 @@ import rice.persistence.*;
 /**
  * @(#) PastImpl.java This is an implementation of the Past interface.
  *
- * @version $Id: PastImpl.java,v 1.44 2005/05/16 19:01:06 amislove Exp $
+ * @version $Id: PastImpl.java 2866 2005-12-20 15:57:54Z jstewart $
  * @author Alan Mislove
  * @author Ansley Post
  * @author Peter Druschel
  */
 public class PastImpl implements Past, Application, ReplicationManagerClient {
 
+  // ----- STATIC FIELDS -----
+  // the number of milliseconds to wait before declaring a message lost
+  /**
+   * DESCRIBE THE FIELD
+   */
+  public final int MESSAGE_TIMEOUT;
+  // = 30000;
+
+  // the percentage of successful replica inserts in order to declare success
+    /**
+   * DESCRIBE THE FIELD
+   */
+  public final double SUCCESSFUL_INSERT_THRESHOLD;
+  // = 0.5;
+
   // ----- VARIABLE FIELDS -----
 
   // this application's endpoint
-  /**
+    /**
    * DESCRIBE THE FIELD
    */
   protected Endpoint endpoint;
@@ -118,12 +136,6 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    */
   protected IdFactory factory;
 
-  // the logger which we will use
-  /**
-   * DESCRIBE THE FIELD
-   */
-  protected Logger log = Logger.getLogger(this.getClass().getName());
-
   // the instance name we are running with
   /**
    * DESCRIBE THE FIELD
@@ -148,23 +160,14 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    */
   public int other = 0;
 
-  // ----- STATIC FIELDS -----
   /**
    * DESCRIBE THE FIELD
    */
-  public final static boolean verbose = false;
-
-  // the number of milliseconds to wait before declaring a message lost
+  protected Environment environment;
   /**
    * DESCRIBE THE FIELD
    */
-  public static int MESSAGE_TIMEOUT = 30000;
-
-  // the percentage of successful replica inserts in order to declare success
-  /**
-   * DESCRIBE THE FIELD
-   */
-  public static double SUCCESSFUL_INSERT_THRESHOLD = 0.5;
+  protected Logger logger;
 
   /**
    * Constructor for Past, using the default policy
@@ -204,7 +207,13 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param trash DESCRIBE THE PARAMETER
    */
   public PastImpl(Node node, StorageManager manager, Cache backup, int replicas, String instance, PastPolicy policy, StorageManager trash) {
-    this.log.setLevel(Level.WARNING);
+    this.environment = node.getEnvironment();
+    logger = environment.getLogManager().getLogger(getClass(), instance);
+    Parameters p = environment.getParameters();
+    MESSAGE_TIMEOUT = p.getInt("p2p_past_messageTimeout");
+    // = 30000;
+    SUCCESSFUL_INSERT_THRESHOLD = p.getDouble("p2p_past_successfulInsertThreshold");
+    // = 0.5;
     this.storage = manager;
     this.backup = backup;
     this.endpoint = node.registerApplication(this, instance);
@@ -225,6 +234,16 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
     this.replicaManager = buildReplicationManager(node, instance);
   }
 
+
+  /**
+   * Gets the Environment attribute of the PastImpl object
+   *
+   * @return The Environment value
+   */
+  public Environment getEnvironment() {
+    return environment;
+  }
+
   /**
    * Returns of the outstanding messages. This is a DEBUGGING method ONLY!
    *
@@ -260,7 +279,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @return A new id
    */
   protected Continuation getResponseContinuation(final PastMessage msg) {
-    log.finer("Getting the Continuation to respond to the message " + msg);
+    if (logger.level <= Logger.FINER) {
+      logger.log("Getting the Continuation to respond to the message " + msg);
+    }
     final ContinuationMessage cmsg = (ContinuationMessage) msg;
 
     return
@@ -326,7 +347,6 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
     return replicationFactor;
   }
 
-
   // ----- UTILITY METHODS -----
 
   /**
@@ -349,6 +369,15 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
     return storage;
   }
 
+  /**
+   * Gets the Instance attribute of the PastImpl object
+   *
+   * @return The Instance value
+   */
+  public String getInstance() {
+    return instance;
+  }
+
   // ----- INTERNAL METHODS -----
 
   /**
@@ -397,7 +426,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param hint DESCRIBE THE PARAMETER
    */
   protected void sendRequest(Id id, PastMessage message, NodeHandle hint, Continuation command) {
-    log.finer("Sending request message " + message + " to id " + id + " via " + hint);
+    if (logger.level <= Logger.FINER) {
+      logger.log("Sending request message " + message + " to id " + id + " via " + hint);
+    }
     CancellableTask timer = endpoint.scheduleMessage(new MessageLostMessage(message.getUID(), getLocalNodeHandle(), id, message, hint), MESSAGE_TIMEOUT);
     insertPending(message.getUID(), timer, command);
     endpoint.route(id, message, hint);
@@ -411,7 +442,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param timer DESCRIBE THE PARAMETER
    */
   private void insertPending(int uid, CancellableTask timer, Continuation command) {
-    log.finer("Loading continuation " + uid + " into pending table");
+    if (logger.level <= Logger.FINER) {
+      logger.log("Loading continuation " + uid + " into pending table");
+    }
     timers.put(new Integer(uid), timer);
     outstanding.put(new Integer(uid), command);
   }
@@ -423,7 +456,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @return The continuation to run
    */
   private Continuation removePending(int uid) {
-    log.finer("Removing and returning continuation " + uid + " from pending table");
+    if (logger.level <= Logger.FINER) {
+      logger.log("Removing and returning continuation " + uid + " from pending table");
+    }
     CancellableTask timer = (CancellableTask) timers.remove(new Integer(uid));
 
     if (timer != null) {
@@ -439,11 +474,13 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param message The message that arrived
    */
   private void handleResponse(PastMessage message) {
-    log.fine("handling reponse message " + message + " from the request");
+    if (logger.level <= Logger.FINE) {
+      logger.log("handling reponse message " + message + " from the request");
+    }
     Continuation command = removePending(message.getUID());
 
     if (command != null) {
-      message.returnResponse(command);
+      message.returnResponse(command, environment, instance);
     }
   }
 
@@ -453,7 +490,7 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param content The content to cache
    */
   private void cache(final PastContent content) {
-    cache(content, new ListenerContinuation("Caching of " + content));
+    cache(content, new ListenerContinuation("Caching of " + content, environment));
   }
 
   /**
@@ -463,7 +500,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param command The command to run once done
    */
   public void cache(final PastContent content, final Continuation command) {
-    log.finer("Inserting PastContent object " + content + " into cache");
+    if (logger.level <= Logger.FINER) {
+      logger.log("Inserting PastContent object " + content + " into cache");
+    }
 
     if ((content != null) && (!content.isMutable())) {
       storage.cache(content.getId(), null, content, command);
@@ -487,7 +526,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
       new StandardContinuation(command) {
         public void receiveResult(Object o) {
           NodeHandleSet replicas = (NodeHandleSet) o;
-          log.finer("Received replicas " + replicas + " for id " + id);
+          if (logger.level <= Logger.FINER) {
+            logger.log("Received replicas " + replicas + " for id " + id);
+          }
 
           // then we send inserts to each replica and wait for at least
           // threshold * num to return successfully
@@ -544,10 +585,12 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param command Command to be performed when the result is received
    */
   public void insert(final PastContent obj, final Continuation command) {
-    log.fine("Inserting the object " + obj + " with the id " + obj.getId());
+    if (logger.level <= Logger.FINER) {
+      logger.log("Inserting the object " + obj + " with the id " + obj.getId());
+    }
 
-    if (PastImpl.verbose) {
-      System.out.println("COUNT: " + System.currentTimeMillis() + " Inserting data of class " + obj.getClass().getName() + " under " + obj.getId().toStringFull());
+    if (logger.level <= Logger.FINEST) {
+      logger.log(" Inserting data of class " + obj.getClass().getName() + " under " + obj.getId().toStringFull());
     }
 
     doInsert(obj.getId(),
@@ -597,8 +640,8 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param command Command to be performed when the result is received
    */
   public void lookup(final Id id, final boolean cache, final Continuation command) {
-    if (PastImpl.verbose) {
-      System.out.println("COUNT: " + System.currentTimeMillis() + " Performing lookup on " + id.toStringFull());
+    if (logger.level <= Logger.FINER) {
+      logger.log(" Performing lookup on " + id.toStringFull());
     }
 
     storage.getObject(id,
@@ -691,17 +734,21 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param command Command to be performed when the result is received
    */
   public void lookupHandles(final Id id, int max, final Continuation command) {
-    log.fine("Retrieving handles of up to " + max + " replicas of the object stored in Past with id " + id);
+    if (logger.level <= Logger.FINE) {
+      logger.log("Retrieving handles of up to " + max + " replicas of the object stored in Past with id " + id);
+    }
 
-    if (PastImpl.verbose) {
-      System.out.println("COUNT: " + System.currentTimeMillis() + " Fetching up to " + max + " handles of " + id.toStringFull());
+    if (logger.level <= Logger.FINER) {
+      logger.log("Fetching up to " + max + " handles of " + id.toStringFull());
     }
 
     getHandles(id, max,
       new StandardContinuation(command) {
         public void receiveResult(Object o) {
           NodeHandleSet replicas = (NodeHandleSet) o;
-          log.finer("Receiving replicas " + replicas + " for lookup Id " + id);
+          if (logger.level <= Logger.FINER) {
+            logger.log("Receiving replicas " + replicas + " for lookup Id " + id);
+          }
 
           MultiContinuation multi =
             new MultiContinuation(parent, replicas.size()) {
@@ -735,7 +782,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param command Command to be performed when the result is received
    */
   public void lookupHandle(Id id, NodeHandle handle, Continuation command) {
-    log.fine("Retrieving handle for id " + id + " from node " + handle);
+    if (logger.level <= Logger.FINE) {
+      logger.log("Retrieving handle for id " + id + " from node " + handle);
+    }
 
     sendRequest(handle, new FetchHandleMessage(getUID(), id, getLocalNodeHandle(), handle.getId()),
       new NamedContinuation("FetchHandleMessage to " + handle + " for " + id, command));
@@ -752,10 +801,12 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param handle DESCRIBE THE PARAMETER
    */
   public void fetch(PastContentHandle handle, Continuation command) {
-    log.fine("Retrieving object associated with content handle " + handle);
+    if (logger.level <= Logger.FINE) {
+      logger.log("Retrieving object associated with content handle " + handle);
+    }
 
-    if (PastImpl.verbose) {
-      System.out.println("COUNT: " + System.currentTimeMillis() + " Fetching object under id " + handle.getId().toStringFull() + " on " + handle.getNodeHandle());
+    if (logger.level <= Logger.FINER) {
+      logger.log("Fetching object under id " + handle.getId().toStringFull() + " on " + handle.getNodeHandle());
     }
 
     NodeHandle han = handle.getNodeHandle();
@@ -784,10 +835,14 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
 
       // if it is a request, look in the cache
       if (!lmsg.isResponse()) {
-        log.finer("Lookup message " + lmsg + " is a request; look in the cache");
+        if (logger.level <= Logger.FINER) {
+          logger.log("Lookup message " + lmsg + " is a request; look in the cache");
+        }
         if (storage.exists(id)) {
           // deliver the message, which will do what we want
-          log.fine("Request for " + id + " satisfied locally - responding");
+          if (logger.level <= Logger.FINE) {
+            logger.log("Request for " + id + " satisfied locally - responding");
+          }
           deliver(endpoint.getId(), lmsg);
           return false;
         }
@@ -797,7 +852,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
 
       if (!lmsg.isResponse()) {
         if (endpoint.replicaSet(lmsg.getId(), lmsg.getMax()).size() == lmsg.getMax()) {
-          log.fine("Hijacking lookup handles request for " + lmsg.getId());
+          if (logger.level <= Logger.FINE) {
+            logger.log("Hijacking lookup handles request for " + lmsg.getId());
+          }
 
           deliver(endpoint.getId(), lmsg);
           return false;
@@ -821,7 +878,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
     if (msg.isResponse()) {
       handleResponse((PastMessage) message);
     } else {
-      log.info("Received message " + message + " with destination " + id);
+      if (logger.level <= Logger.INFO) {
+        logger.log("Received message " + message + " with destination " + id);
+      }
 
       if (msg instanceof InsertMessage) {
         final InsertMessage imsg = (InsertMessage) msg;
@@ -854,7 +913,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
         storage.getObject(lmsg.getId(),
           new StandardContinuation(getResponseContinuation(lmsg)) {
             public void receiveResult(Object o) {
-              log.fine("Received object " + o + " for id " + lmsg.getId());
+              if (logger.level <= Logger.FINE) {
+                logger.log("Received object " + o + " for id " + lmsg.getId());
+              }
 
               // send result back
               parent.receiveResult(o);
@@ -864,7 +925,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
                 (o != null) &&
                 (!((PastContent) o).isMutable())) {
                 NodeHandle handle = lmsg.getPreviousNodeHandle();
-                log.fine("Pushing cached copy of " + ((PastContent) o).getId() + " to " + handle);
+                if (logger.level <= Logger.FINE) {
+                  logger.log("Pushing cached copy of " + ((PastContent) o).getId() + " to " + handle);
+                }
 
                 CacheMessage cmsg = new CacheMessage(getUID(), (PastContent) o, getLocalNodeHandle(), handle.getId());
                 //endpoint.route(null, cmsg, handle);
@@ -874,7 +937,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
       } else if (msg instanceof LookupHandlesMessage) {
         LookupHandlesMessage lmsg = (LookupHandlesMessage) msg;
         NodeHandleSet set = endpoint.replicaSet(lmsg.getId(), lmsg.getMax());
-        log.finer("Returning replica set " + set + " for lookup handles of id " + lmsg.getId() + " max " + lmsg.getMax() + " at " + endpoint.getId());
+        if (logger.level <= Logger.FINER) {
+          logger.log("Returning replica set " + set + " for lookup handles of id " + lmsg.getId() + " max " + lmsg.getMax() + " at " + endpoint.getId());
+        }
         getResponseContinuation(msg).receiveResult(set);
       } else if (msg instanceof FetchMessage) {
         FetchMessage fmsg = (FetchMessage) msg;
@@ -891,7 +956,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
               PastContent content = (PastContent) o;
 
               if (content != null) {
-                log.fine("Retrieved data for fetch handles of id " + fmsg.getId());
+                if (logger.level <= Logger.FINE) {
+                  logger.log("Retrieved data for fetch handles of id " + fmsg.getId());
+                }
                 parent.receiveResult(content.getHandle(PastImpl.this));
               } else {
                 parent.receiveResult(null);
@@ -901,7 +968,9 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
       } else if (msg instanceof CacheMessage) {
         cache(((CacheMessage) msg).getContent());
       } else {
-        log.severe("ERROR - Received message " + msg + "of unknown type.");
+        if (logger.level <= Logger.SEVERE) {
+          logger.log("ERROR - Received message " + msg + "of unknown type.");
+        }
       }
     }
   }
@@ -931,19 +1000,27 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    * @param command DESCRIBE THE PARAMETER
    */
   public void fetch(final Id id, NodeHandle hint, Continuation command) {
-    log.finer("Sending out replication fetch request for the id " + id);
+    if (logger.level <= Logger.FINER) {
+      logger.log("Sending out replication fetch request for the id " + id);
+    }
 
     policy.fetch(id, hint, backup, this,
       new StandardContinuation(command) {
         public void receiveResult(Object o) {
           if (o == null) {
-            log.warning("Could not fetch id " + id + " - policy returned null in namespace " + instance);
+            if (logger.level <= Logger.WARNING) {
+              logger.log("Could not fetch id " + id + " - policy returned null in namespace " + instance);
+            }
             parent.receiveResult(new Boolean(false));
           } else {
-            log.finest("inserting replica of id " + id);
+            if (logger.level <= Logger.FINEST) {
+              logger.log("inserting replica of id " + id);
+            }
 
             if (!(o instanceof PastContent)) {
-              System.err.println("ERROR! Not PastContent " + o.getClass().getName() + " " + o);
+              if (logger.level <= Logger.WARNING) {
+                logger.log("ERROR! Not PastContent " + o.getClass().getName() + " " + o);
+              }
             }
             storage.getStorage().store(((PastContent) o).getId(), null, (PastContent) o, parent);
           }
@@ -1011,10 +1088,59 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
     return storage.getStorage().exists(id);
   }
 
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param id DESCRIBE THE PARAMETER
+   * @param command DESCRIBE THE PARAMETER
+   */
+  public void existsInOverlay(Id id, Continuation command) {
+    lookupHandles(id, replicationFactor + 1,
+      new StandardContinuation(command) {
+        public void receiveResult(Object result) {
+          Object results[] = (Object[]) result;
+          for (int i = 0; i < results.length; i++) {
+            if (results[i] instanceof PastContentHandle) {
+              parent.receiveResult(Boolean.TRUE);
+              return;
+            }
+          }
+          parent.receiveResult(Boolean.FALSE);
+        }
+      });
+  }
+
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param id DESCRIBE THE PARAMETER
+   * @param command DESCRIBE THE PARAMETER
+   */
+  public void reInsert(Id id, Continuation command) {
+    storage.getObject(id,
+      new StandardContinuation(command) {
+        public void receiveResult(final Object o) {
+          insert((PastContent) o,
+            new StandardContinuation(parent) {
+              public void receiveResult(Object result) {
+                Boolean results[] = (Boolean[]) result;
+                for (int i = 0; i < results.length; i++) {
+                  if (results[i].booleanValue()) {
+                    parent.receiveResult(Boolean.TRUE);
+                    return;
+                  }
+                }
+                parent.receiveResult(Boolean.FALSE);
+              }
+            });
+        }
+      });
+  }
+
   /**
    * Class which builds a message
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   public interface MessageBuilder {
@@ -1025,4 +1151,5 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
      */
     public PastMessage buildMessage();
   }
+
 }
