@@ -38,6 +38,7 @@ package rice.p2p.past.testing;
 
 import rice.*;
 
+import rice.environment.Environment;
 import rice.p2p.commonapi.*;
 import rice.p2p.commonapi.testing.*;
 import rice.p2p.past.*;
@@ -54,7 +55,7 @@ import java.io.Serializable;
 /**
  * @(#) PastRegrTest.java Provides regression testing for the Past service
  *
- * @version $Id: PastRegrTest.java,v 1.9 2005/03/21 20:26:08 amislove Exp $
+ * @version $Id: PastRegrTest.java 2844 2005-12-15 13:07:05Z jeffh $
  * @author Alan Mislove
  */
 
@@ -72,11 +73,10 @@ public class PastRegrTest extends CommonAPITest {
    */
   protected PastImpl pasts[];
 
-  // a random number generator
   /**
    * DESCRIBE THE FIELD
    */
-  protected Random rng;
+  protected boolean running = true;
 
   // the instance name to use
   /**
@@ -92,24 +92,29 @@ public class PastRegrTest extends CommonAPITest {
 
   /**
    * Constructor which sets up all local variables
+   *
+   * @param env DESCRIBE THE PARAMETER
+   * @exception IOException DESCRIBE THE EXCEPTION
    */
-  public PastRegrTest() {
+  public PastRegrTest(Environment env) throws IOException {
+    super(env);
     pasts = new PastImpl[NUM_NODES];
     storages = new StorageManager[NUM_NODES];
-    rng = new Random();
 
-      new Thread() {
-        public void run() {
-          while (true) {
-            try {
-              sleep(50);
-              simulate();
-            } catch (Exception e) {
-              System.out.println(e + " blah");
+    if (PROTOCOL == PROTOCOL_DIRECT) {
+        new Thread() {
+          public void run() {
+            while (running) {
+              try {
+                sleep(50);
+                simulate();
+              } catch (Exception e) {
+                System.out.println(e + " blah");
+              }
             }
           }
-        }
-      }.start();
+        }.start();
+    }
   }
 
   /**
@@ -121,8 +126,8 @@ public class PastRegrTest extends CommonAPITest {
   protected void processNode(int num, Node node) {
     try {
       storages[num] = new StorageManagerImpl(FACTORY,
-        new PersistentStorage(FACTORY, "root-" + num, ".", 1000000),
-        new LRUCache(new MemoryStorage(FACTORY), 100000));
+        new PersistentStorage(FACTORY, "root-" + num, ".", 1000000, environment),
+        new LRUCache(new MemoryStorage(FACTORY), 100000, environment));
       pasts[num] = new PastImpl(node, storages[num], REPLICATION_FACTOR, INSTANCE);
     } catch (IOException e) {
       throw new RuntimeException(e);
@@ -150,8 +155,8 @@ public class PastRegrTest extends CommonAPITest {
    * Tests routing a Past request to a particular node.
    */
   protected void testRouteRequest() {
-    final PastImpl local = pasts[rng.nextInt(NUM_NODES)];
-    final PastImpl remote = pasts[rng.nextInt(NUM_NODES)];
+    final PastImpl local = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
+    final PastImpl remote = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
     final Id remoteId = remote.getLocalNodeHandle().getId();
     final PastContent file = new TestPastContent(remoteId);
 
@@ -229,8 +234,8 @@ public class PastRegrTest extends CommonAPITest {
    * Tests overwriting an exiting object with a new one
    */
   protected void testVersionControl() {
-    final PastImpl local = pasts[rng.nextInt(NUM_NODES)];
-    final PastImpl remote = pasts[rng.nextInt(NUM_NODES)];
+    final PastImpl local = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
+    final PastImpl remote = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
     final Id remoteId = remote.getLocalNodeHandle().getId();
     final PastContent oldFile = new VersionedTestPastContent(remoteId, 0);
     final PastContent newFile = new VersionedTestPastContent(remoteId, 1);
@@ -358,55 +363,56 @@ public class PastRegrTest extends CommonAPITest {
 
   /**
    * Tests the parameter checking in Past. protected void testParameterChecks()
-   * { final PastImpl local = pasts[rng.nextInt(NUM_NODES)]; final Id localId =
-   * local.getLocalNodeHandle().getId(); sectionStart("Parameter Checks
-   * Testing"); / Null insert should fail stepStart("Insertion Of Null");
-   * local.insert(null, new TestExceptionCommand() { public void receive(Object
-   * result) throws Exception { assertTrue("Exception returned should not be
-   * null", result != null); assertTrue("Exception should be return", result
-   * instanceof Exception); stepDone(); / Null lookup should fail
-   * stepStart("Lookup Of Null"); local.lookup(null, new TestExceptionCommand()
-   * { public void receive(Object result) throws Exception {
-   * assertTrue("Exception returned should not be null", result != null);
-   * assertTrue("Exception should be return", result instanceof Exception);
-   * stepDone(); / Null fetch should fail stepStart("Fetch Of Null");
-   * local.fetch((PastContentHandle) null, new TestExceptionCommand() { public
-   * void receive(Object result) throws Exception { assertTrue("Exception
-   * returned should not be null", result != null); assertTrue("Exception should
-   * be return", result instanceof Exception); stepDone(); / Null lookup handles
-   * should fail stepStart("Lookup Handles Of Null"); local.lookupHandles(null,
-   * 1, new TestExceptionCommand() { public void receive(Object result) throws
-   * Exception { assertTrue("Exception returned should not be null", result !=
-   * null); assertTrue("Exception should be return", result instanceof
-   * Exception); stepDone(); / Lookup handles of -4 should fail
-   * stepStart("Lookup Handles Of -4"); local.lookupHandles(localId, -4, new
+   * { final PastImpl local = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
+   * final Id localId = local.getLocalNodeHandle().getId();
+   * sectionStart("Parameter Checks Testing"); / Null insert should fail
+   * stepStart("Insertion Of Null"); local.insert(null, new
    * TestExceptionCommand() { public void receive(Object result) throws
    * Exception { assertTrue("Exception returned should not be null", result !=
    * null); assertTrue("Exception should be return", result instanceof
-   * Exception); stepDone(); / Lookup handles of 0 should fail stepStart("Lookup
-   * Handles Of 0"); local.lookupHandles(localId, 0, new TestExceptionCommand()
+   * Exception); stepDone(); / Null lookup should fail stepStart("Lookup Of
+   * Null"); local.lookup(null, new TestExceptionCommand() { public void
+   * receive(Object result) throws Exception { assertTrue("Exception returned
+   * should not be null", result != null); assertTrue("Exception should be
+   * return", result instanceof Exception); stepDone(); / Null fetch should fail
+   * stepStart("Fetch Of Null"); local.fetch((PastContentHandle) null, new
+   * TestExceptionCommand() { public void receive(Object result) throws
+   * Exception { assertTrue("Exception returned should not be null", result !=
+   * null); assertTrue("Exception should be return", result instanceof
+   * Exception); stepDone(); / Null lookup handles should fail stepStart("Lookup
+   * Handles Of Null"); local.lookupHandles(null, 1, new TestExceptionCommand()
    * { public void receive(Object result) throws Exception {
    * assertTrue("Exception returned should not be null", result != null);
    * assertTrue("Exception should be return", result instanceof Exception);
-   * stepDone(); sectionDone(); testFetch(); } }); simulate(); } }); simulate();
-   * } }); simulate(); } }); simulate(); } }); simulate(); } }); simulate(); }
+   * stepDone(); / Lookup handles of -4 should fail stepStart("Lookup Handles Of
+   * -4"); local.lookupHandles(localId, -4, new TestExceptionCommand() { public
+   * void receive(Object result) throws Exception { assertTrue("Exception
+   * returned should not be null", result != null); assertTrue("Exception should
+   * be return", result instanceof Exception); stepDone(); / Lookup handles of 0
+   * should fail stepStart("Lookup Handles Of 0"); local.lookupHandles(localId,
+   * 0, new TestExceptionCommand() { public void receive(Object result) throws
+   * Exception { assertTrue("Exception returned should not be null", result !=
+   * null); assertTrue("Exception should be return", result instanceof
+   * Exception); stepDone(); sectionDone(); testFetch(); } }); simulate(); } });
+   * simulate(); } }); simulate(); } }); simulate(); } }); simulate(); } });
+   * simulate(); }
    */
 
   /**
    * Tests the fetch function in Past.
    */
   protected void testFetch() {
-    final PastImpl local = pasts[rng.nextInt(NUM_NODES)];
-    final PastImpl remote1 = pasts[rng.nextInt(NUM_NODES)];
-    PastImpl tmp = pasts[rng.nextInt(NUM_NODES)];
+    final PastImpl local = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
+    final PastImpl remote1 = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
+    PastImpl tmp = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
 
     while (tmp == remote1) {
-      tmp = pasts[rng.nextInt(NUM_NODES)];
+      tmp = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
     }
 
     final PastImpl remote2 = tmp;
 
-    final Id id = pasts[rng.nextInt(NUM_NODES)].getLocalNodeHandle().getId();
+    final Id id = pasts[environment.getRandomSource().nextInt(NUM_NODES)].getLocalNodeHandle().getId();
     final PastContent file1 = new VersionedTestPastContent(id, 1);
     final PastContent file2 = new VersionedTestPastContent(id, 2);
 
@@ -508,8 +514,8 @@ public class PastRegrTest extends CommonAPITest {
    * Tests the lookup handles function in Past.
    */
   protected void testLookupHandles() {
-    final PastImpl local = pasts[rng.nextInt(NUM_NODES)];
-    final PastImpl remote = pasts[rng.nextInt(NUM_NODES)];
+    final PastImpl local = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
+    final PastImpl remote = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
     final Id remoteId = remote.getLocalNodeHandle().getId();
     final PastContent file = new TestPastContent(remoteId);
 
@@ -578,8 +584,8 @@ public class PastRegrTest extends CommonAPITest {
                       stepDone();
 
                       // Check file exists (at a huge number of replicas)
-                      stepStart("Remote Handles Lookup - 500 Replicas");
-                      local.lookupHandles(remoteId, 500,
+                      stepStart("Remote Handles Lookup - 12 Replicas");
+                      local.lookupHandles(remoteId, 12,
                         new TestCommand() {
                           public void receive(Object result) throws Exception {
                             assertTrue("Replicas should not be null", result != null);
@@ -632,7 +638,7 @@ public class PastRegrTest extends CommonAPITest {
    * Tests the dynamic caching function in Past.
    */
   protected void testCaching() {
-    final PastImpl local = pasts[rng.nextInt(NUM_NODES)];
+    final PastImpl local = pasts[environment.getRandomSource().nextInt(NUM_NODES)];
     final Id id1 = generateId();
     final Id id2 = generateId();
     final PastContent file1 = new TestPastContent(id1);
@@ -712,6 +718,8 @@ public class PastRegrTest extends CommonAPITest {
                             stepDone();
 
                             sectionDone();
+
+                            cleanUp();
                           }
                         });
                       simulate();
@@ -744,27 +752,37 @@ public class PastRegrTest extends CommonAPITest {
    */
   private Id generateId() {
     byte[] data = new byte[20];
-    rng.nextBytes(data);
+    environment.getRandomSource().nextBytes(data);
     return FACTORY.buildId(data);
   }
 
+  /**
+   * DESCRIBE THE METHOD
+   */
+  protected void cleanUp() {
+    running = false;
+    environment.destroy();
+  }
+
 
   /**
    * Usage: DistPastTest [-port p] [-bootstrap host[:port]] [-nodes n]
-   * [-protocol (rmi|wire)] [-help]
+   * [-protocol (direct|socket)] [-help]
    *
    * @param args DESCRIBE THE PARAMETER
+   * @exception IOException DESCRIBE THE EXCEPTION
    */
-  public static void main(String args[]) {
-    parseArgs(args);
-    PastRegrTest pastTest = new PastRegrTest();
+  public static void main(String args[]) throws IOException {
+//    System.setOut(new PrintStream(new FileOutputStream("pastrtest.txt")));
+    Environment env = parseArgs(args);
+    PastRegrTest pastTest = new PastRegrTest(env);
     pastTest.start();
   }
 
   /**
    * Common superclass for test commands.
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected class TestCommand implements Continuation {
@@ -803,7 +821,7 @@ public class PastRegrTest extends CommonAPITest {
   /**
    * Common superclass for test commands which should throw an exception
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected class TestExceptionCommand implements Continuation {
@@ -842,7 +860,7 @@ public class PastRegrTest extends CommonAPITest {
   /**
    * Utility class for past content objects
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected static class TestPastContent implements PastContent {
@@ -934,7 +952,7 @@ public class PastRegrTest extends CommonAPITest {
   /**
    * DESCRIBE THE CLASS
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected static class VersionedTestPastContent extends TestPastContent {
@@ -983,7 +1001,7 @@ public class PastRegrTest extends CommonAPITest {
   /**
    * DESCRIBE THE CLASS
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected static class NonOverwritingTestPastContent extends VersionedTestPastContent {
@@ -1014,7 +1032,7 @@ public class PastRegrTest extends CommonAPITest {
   /**
    * DESCRIBE THE CLASS
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected static class NonMutableTestPastContent extends TestPastContent {
@@ -1055,7 +1073,7 @@ public class PastRegrTest extends CommonAPITest {
   /**
    * Utility class for past content object handles
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected static class TestPastContentHandle implements PastContentHandle {
@@ -1103,7 +1121,7 @@ public class PastRegrTest extends CommonAPITest {
   /**
    * Utility class which simulates a route message
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected static class TestRouteMessage implements RouteMessage {
