@@ -41,6 +41,8 @@ import java.lang.Comparable;
 import java.lang.ref.*;
 import java.util.*;
 
+import rice.environment.random.RandomSource;
+
 /**
  * Represents a Pastry identifier for a node, object or key. A single identifier
  * and the bit length for Ids is stored in this class. Ids are stored little
@@ -48,7 +50,7 @@ import java.util.*;
  * Ids are to be constructed from the build() methods, which ensure that only
  * one copy of each Id is in memory at a time.
  *
- * @version $Id: Id.java,v 1.25 2005/03/11 00:58:11 jeffh Exp $
+ * @version $Id: Id.java 2634 2005-07-07 21:18:42Z jeffh $
  * @author Andrew Ladd
  * @author Peter Druschel
  * @author Alan Mislove
@@ -480,7 +482,6 @@ public class Id implements rice.p2p.commonapi.Id {
     }
 
     Distance d = new Distance(dist);
-    //System.out.println("Dist:" + this + nid + d);
 
     return d;
   }
@@ -499,7 +500,6 @@ public class Id implements rice.p2p.commonapi.Id {
     }
 
     Distance d = new Distance(dist);
-    //System.out.println("Diff:" + this + nid + d);
 
     return d;
   }
@@ -679,6 +679,22 @@ public class Id implements rice.p2p.commonapi.Id {
     return buffer.toString();
   }
 
+  /**
+   * Similar to toString(), but not wrapped by <0x ..>
+   *
+   * @return
+   */
+  public String toStringBare() {
+    StringBuffer buffer = new StringBuffer();
+
+    int n = IdBitLength / 4;
+    for (int i = n - 1; i >= n - 6; i--) {
+      buffer.append(tran[getDigit(i, 4)]);
+    }
+
+    return buffer.toString();
+  }
+
   /**
    * Returns the complete represntation of this Id, in hex.
    *
@@ -782,7 +798,6 @@ public class Id implements rice.p2p.commonapi.Id {
       }
     }
 
-    //System.out.println("absDist=" + new Distance(dist));
     return dist;
   }
 
@@ -827,6 +842,18 @@ public class Id implements rice.p2p.commonapi.Id {
     return build(material);
   }
 
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param rng DESCRIBE THE PARAMETER
+   * @return DESCRIBE THE RETURN VALUE
+   */
+  public static Id makeRandomId(RandomSource rng) {
+    byte material[] = new byte[IdBitLength / 8];
+    rng.nextBytes(material);
+    return build(material);
+  }
+
 
   // ----- SUPPORT FOR ID COALESCING -----
 
@@ -970,7 +997,7 @@ public class Id implements rice.p2p.commonapi.Id {
    * A class for representing and manipulating the distance between two Ids on
    * the circle.
    *
-   * @version $Id: Id.java,v 1.25 2005/03/11 00:58:11 jeffh Exp $
+   * @version $Id: Id.java 2634 2005-07-07 21:18:42Z jeffh $
    * @author amislove
    */
   public static class Distance implements rice.p2p.commonapi.Id.Distance {
