@@ -48,7 +48,7 @@ import java.io.*;
  * strictly a set: when the ring is small, a node may appear in both the cw and
  * the ccw half of the "set".
  *
- * @version $Id: LeafSet.java,v 1.45 2005/03/11 00:58:04 jeffh Exp $
+ * @version $Id: LeafSet.java 2819 2005-11-28 12:32:42Z jeffh $
  * @author Andrew Ladd
  * @author Peter Druschel
  */
@@ -131,41 +131,12 @@ public class LeafSet extends Observable implements Serializable {
   }
 
   /**
-   * Finds the NodeHandle associated with the NodeId.
+   * Gets the Index attribute of the LeafSet object
    *
    * @param nh DESCRIBE THE PARAMETER
-   * @return the handle associated with that id or null if no such handle is
-   *      found.
+   * @return The Index value
    * @exception NoSuchElementException DESCRIBE THE EXCEPTION
    */
-//  public NodeHandle get(NodeId nid, int foo)
-//  {
-//    NodeHandle res = cwSet.get(nid);
-//    if (res != null) return res;
-//    return ccwSet.get(nid);
-//  }
-
-
-  /**
-   * Gets the index of the element with the given node id.
-   *
-   * @param nh DESCRIBE THE PARAMETER
-   * @return the index or throws a NoSuchElementException
-   * @exception NoSuchElementException DESCRIBE THE EXCEPTION
-   */
-//  public int getIndex(NodeId nid, int foo) throws NoSuchElementException {
-//    int index;
-//
-//    if (baseId.equals(nid)) return 0;
-//
-//    index = cwSet.getIndex(nid);
-//    if (index >= 0) return index + 1;
-//    index = ccwSet.getIndex(nid);
-//    if (index >= 0) return -index - 1;
-//
-//    throw new NoSuchElementException();
-//  }
-
   public int getIndex(NodeHandle nh) throws NoSuchElementException {
     int index;
 
@@ -208,16 +179,22 @@ public class LeafSet extends Observable implements Serializable {
    *
    * @return the number of unique nodes in the leafset
    */
-  private int getUniqueCount() {
-    Vector v = new Vector();
-
-    for (int i = -ccwSize(); i <= cwSize(); i++) {
-      if (!v.contains(get(i).getNodeId())) {
-        v.add(get(i).getNodeId());
-      }
-    }
-
-    return v.size();
+  public int getUniqueCount() {
+    HashSet superset = new HashSet();
+    superset.addAll(cwSet.getCollection());
+    superset.addAll(ccwSet.getCollection());
+    return superset.size() + 1;
+    // the local node
+//
+//    Vector v = new Vector();
+//
+//    for (int i=-ccwSize(); i<=cwSize(); i++) {
+//      if (! v.contains(get(i).getNodeId())) {
+//        v.add(get(i).getNodeId());
+//      }
+//    }
+//
+//    return v.size();
   }
 
   /**
@@ -312,40 +289,15 @@ public class LeafSet extends Observable implements Serializable {
    * @return the node handle removed or null if nothing.
    */
   public NodeHandle remove(NodeHandle nh) {
-    //System.out.println("Removing " + nid + " from " + this);
-
     NodeHandle res1 = cwSet.remove(nh);
     NodeHandle res2 = ccwSet.remove(nh);
     if (res1 != null) {
-      //System.out.println("removed " + res1.getNodeId());
       return res1;
     } else {
-      //System.out.println("removed " + res2.getNodeId());
       return res2;
     }
   }
 
-  /**
-   * Removes a node id and its handle from the set.
-   *
-   * @return the node handle removed or null if nothing.
-   */
-//  public NodeHandle remove(NodeId nid)
-//  {
-//    //System.out.println("Removing " + nid + " from " + this);
-//
-//    NodeHandle res1 = cwSet.remove(nid);
-//    NodeHandle res2 = ccwSet.remove(nid);
-//    if (res1 != null) {
-//      //System.out.println("removed " + res1.getNodeId());
-//      return res1;
-//    }
-//    else {
-//      //System.out.println("removed " + res2.getNodeId());
-//      return res2;
-//    }
-//  }
-
   /**
    * Gets the maximal size of the leaf set.
    *
@@ -356,12 +308,21 @@ public class LeafSet extends Observable implements Serializable {
   }
 
   /**
-   * Gets the current size of the leaf set.
+   * Gets the current size of the leaf set. Note that if the leafset overlaps,
+   * there will be duplicates. If you want the unique nodes, use
+   * getUniqueCount().
    *
    * @return the size.
    */
   public int size() {
-    return cwSet.size() + ccwSet.size();
+    return cwSize() + ccwSize();
+    // old code
+
+    // 8.16.5 decided not to do this because it will break too much code
+//    HashSet superset = new HashSet();
+//    superset.addAll(cwSet.getCollection());
+//    superset.addAll(ccwSet.getCollection());
+//    return superset.size();
   }
 
   /**
@@ -443,8 +404,6 @@ public class LeafSet extends Observable implements Serializable {
       cwMS = cwSet.mostSimilar(nid);
     }
 
-    //System.out.println("cwMS=" + cwMS + " ccwMS=" + ccwMS);
-
     cwMinDist = cwSet.get(cwMS).getNodeId().distance(nid);
     ccwMinDist = ccwSet.get(ccwMS).getNodeId().distance(nid);
 
@@ -507,8 +466,6 @@ public class LeafSet extends Observable implements Serializable {
    * @return the ordered set of nodehandles
    */
   public NodeSet replicaSet(Id key, int max) {
-    //System.out.println("replicaSet Called with max= " + max + " key= " + key);
-    //System.out.println(this);
     NodeSet set = new NodeSet();
     if (max < 1) {
       return set;
@@ -523,8 +480,6 @@ public class LeafSet extends Observable implements Serializable {
 
     // compute the nearest node
     int nearest = mostSimilar(key);
-    //System.out.println("nearest=" + nearest + " key=" + key);
-
 
     // add the key's root
     set.put(get(nearest));
@@ -604,10 +559,10 @@ public class LeafSet extends Observable implements Serializable {
   public IdRange range(NodeHandle n, int r) {
     // first, we check the arguments
     if (r < 0) {
-      return null;
+      throw new IllegalArgumentException("Range must be greater than or equal to zero. Attempted " + r);
     }
-    if (!(member(n) || baseId.equals(n.getNodeId()))) {
-      return null;
+    if (!(member(n) || baseHandle.equals(n))) {
+      throw new RangeCannotBeDeterminedException("Node " + n + " is not in this leafset.", r, Integer.MIN_VALUE, getUniqueCount(), n, this);
     }
 
     // get the position of the node and the number of nodes in the network
@@ -615,6 +570,7 @@ public class LeafSet extends Observable implements Serializable {
     int num = getUniqueCount();
     NodeHandle cw;
     NodeHandle ccw;
+    // these are the nodes in the Leafset that represent the extents of the range
 
     // if this leafset overlaps (or we are the only node in the network), then we have
     // global knowledge about the network and can determine any range.  Otherwise, we
@@ -647,7 +603,7 @@ public class LeafSet extends Observable implements Serializable {
 
     // if either of it's pair nodes are null, then we cannot determine the range
     if ((ccw == null) || (cw == null)) {
-      return null;
+      throw new RangeCannotBeDeterminedException("This leafset doesn't have enough information to provide the correct range.", r, pos, getUniqueCount(), n, this);
     }
 
     // otherwise, we then construct the ranges which comprise the main range, and finally
@@ -674,8 +630,15 @@ public class LeafSet extends Observable implements Serializable {
    */
   public IdRange range(NodeHandle n, int r, boolean cw) {
     IdRange rr = range(n, r);
-    IdRange rprev = range(n, r - 1);
-
+    if (r == 0) {
+      return rr;
+    }
+    IdRange rprev = null;
+    try {
+      rprev = range(n, r - 1);
+    } catch (RangeCannotBeDeterminedException rcbde) {
+      // keeps previous functionality now that we are throwing rcbde rather than returning null
+    }
     if (rr == null || rprev == null) {
       return rr;
     }
@@ -688,7 +651,6 @@ public class LeafSet extends Observable implements Serializable {
     } else {
       res = rprev.diff(rr);
     }
-    //System.out.println("(" + rr + ").diff(" + rprev + ")=" + res);
     return res;
   }
 
@@ -709,10 +671,8 @@ public class LeafSet extends Observable implements Serializable {
   public boolean merge(LeafSet remotels, NodeHandle from, RoutingTable routeTable,
                        PastrySecurityManager security, boolean testOnly, Set insertedHandles) {
 
-    //System.out.println("LeafSet::merge of " + remotels + " into \n" + this);
     boolean changed;
 
-    //System.out.println("LeafSet::merge of " + remotels + " into \n" + this);
     boolean result = false;
     int cwSize = remotels.cwSize();
     int ccwSize = remotels.ccwSize();
@@ -739,7 +699,6 @@ public class LeafSet extends Observable implements Serializable {
           NodeId closestId = remotels.get(closest).getNodeId();
 
           if (closest == -remotels.ccwSize() || closest == remotels.cwSize()) {
-            //System.out.println("LeafSet::merge at " + this + " received an out of range leafset from " + remotels);
             // doesn't hurt to merge it anyways
             //return;
           }
@@ -794,8 +753,6 @@ public class LeafSet extends Observable implements Serializable {
       }
     }
 
-    //System.out.println("LeafSet::merge cw=" + cw + " ccw=" + ccw);
-
     for (int i = cw; i <= cwSize; i++) {
       NodeHandle nh;
 
@@ -891,8 +848,6 @@ public class LeafSet extends Observable implements Serializable {
       }
     }
 
-    //System.out.println("LeafSet::merge result: " + this);
-
     return result;
   }
 
