@@ -36,68 +36,85 @@ if advised of the possibility of such damage.
 package rice.pastry.socket;
 
 import java.io.*;
+import java.lang.ref.WeakReference;
 import java.net.*;
 import java.nio.*;
 import java.nio.channels.*;
 import java.util.*;
 
+import rice.environment.logging.Logger;
+import rice.environment.params.Parameters;
+import rice.environment.random.RandomSource;
 import rice.pastry.*;
 import rice.pastry.messaging.*;
 import rice.pastry.routing.*;
 import rice.pastry.socket.messaging.*;
 import rice.selector.*;
+import rice.selector.TimerTask;
 
 /**
  * Class which keeps track of the best routes to remote nodes. This class is
  * also therefore in charge of declaring node death and liveness.
  *
- * @version $Id: SocketSourceRouteManager.java,v 1.15 2005/05/10 20:53:51
- *      amislove Exp $
+ * @version $Id: SocketSourceRouteManager.java 2901 2006-01-10 15:22:03Z jeffh $
  * @author Alan Mislove
  */
 public class SocketSourceRouteManager {
 
-  // the local pastry node
-  private SocketPastryNode spn;
-
-  // the list of best routes
-  private Hashtable managers;
-
-  // the socket manager below this manager
-  private SocketCollectionManager manager;
-
-  // the node hanlde pool
-  private SocketNodeHandlePool pool;
-
-  // the address of this local node
-  private EpochInetSocketAddress localAddress;
-
   // the minimum amount of time between check dead checks on dead routes
   /**
    * DESCRIBE THE FIELD
    */
-  public static long CHECK_DEAD_THROTTLE = 1000 * 60 * 5;
+  public long CHECK_DEAD_THROTTLE;
 
   // the minimum amount of time between pings
   /**
    * DESCRIBE THE FIELD
    */
-  public static long PING_THROTTLE = 1000 * 60 * 5;
+  public long PING_THROTTLE;
+
+  // the local pastry node
+  private SocketPastryNode spn;
+
+  // the socket manager below this manager
+  private SocketCollectionManager manager;
+
+  // the address of this local node
+  private EpochInetSocketAddress localAddress;
+
+  private Logger logger;
+
+  /**
+   * Invariant: if an AM has a message in its queue, it is in here, if not, it
+   * isn't. This prevents outgoing messages from being lost.
+   */
+  HashSet hardLinks = new HashSet();
+
+  /**
+   * EpochInetSocketAddress -> WeakReference(NodeHandle) Note that it is
+   * critical to keep the key == NodeHandle.eaddress. And I mean the same
+   * object!!! not .equals(). The whole memory management will get confused if
+   * this is not the case.
+   */
+  WeakHashMap nodeHandles = new WeakHashMap();
 
   /**
    * Constructor
    *
    * @param node The local node
-   * @param pool The node hanlde pool
    * @param bindAddress The address which the node should bind to
    * @param proxyAddress The address which the node should advertise as it's
    *      address
+   * @param random DESCRIBE THE PARAMETER
    */
-  protected SocketSourceRouteManager(SocketPastryNode node, SocketNodeHandlePool pool, EpochInetSocketAddress bindAddress, EpochInetSocketAddress proxyAddress) {
+  protected SocketSourceRouteManager(SocketPastryNode node, EpochInetSocketAddress bindAddress, EpochInetSocketAddress proxyAddress, RandomSource random) {
     this.spn = node;
-    this.pool = pool;
-    this.managers = new Hashtable();
-    this.manager = new SocketCollectionManager(node, pool, this, bindAddress, proxyAddress);
+    Parameters p = node.getEnvironment().getParameters();
+    CHECK_DEAD_THROTTLE = p.getLong("pastry_socket_srm_check_dead_throttle");
+    PING_THROTTLE = p.getLong("pastry_socket_srm_ping_throttle");
+
+    this.logger = node.getEnvironment().getLogManager().getLogger(SocketSourceRouteManager.class, null);
+    this.manager = new SocketCollectionManager(node, this, bindAddress, proxyAddress, random);
     this.localAddress = bindAddress;
   }
 
@@ -109,16 +126,26 @@ public class SocketSourceRouteManager {
   public HashMap getBest() {
     HashMap result = new HashMap();
 
-    Iterator i = managers.keySet().iterator();
+    synchronized (nodeHandles) {
+      Iterator i = nodeHandles.keySet().iterator();
 
-    while (i.hasNext()) {
-      Object addr = i.next();
+      while (i.hasNext()) {
+        Object addr = i.next();
 
-      if (((AddressManager) managers.get(addr)).getLiveness() < SocketNodeHandle.LIVENESS_DEAD) {
-        result.put(addr, ((AddressManager) managers.get(addr)).best);
+        WeakReference wr = (WeakReference) nodeHandles.get(addr);
+        if (wr != null) {
+          SocketNodeHandle snh = (SocketNodeHandle) wr.get();
+          if (snh != null) {
+            AddressManager am = snh.addressManager;
+            if (am != null) {
+              if (am.getLiveness() < SocketNodeHandle.LIVENESS_DEAD) {
+                result.put(addr, am.best);
+              }
+            }
+          }
+        }
       }
     }
-
     return result;
   }
 
@@ -140,14 +167,58 @@ public class SocketSourceRouteManager {
    * @return The AddressManager value
    */
   protected AddressManager getAddressManager(EpochInetSocketAddress address, boolean search) {
-    AddressManager manager = (AddressManager) managers.get(address);
+    synchronized (nodeHandles) {
+      AddressManager manager = getAddressManager(address);
 
-    if (manager == null) {
-      manager = new AddressManager(address, search);
-      managers.put(address, manager);
+      if (manager == null) {
+        manager = putAddressManager(address, search);
+      }
+
+      return manager;
+    }
+  }
+
+  /**
+   * Gets the NodeHandle attribute of the SocketSourceRouteManager object
+   *
+   * @param address DESCRIBE THE PARAMETER
+   * @return The NodeHandle value
+   */
+  public SocketNodeHandle getNodeHandle(EpochInetSocketAddress address) {
+    synchronized (nodeHandles) {
+      WeakReference wr = (WeakReference) nodeHandles.get(address);
+      if (wr == null) {
+        return null;
+      }
+
+      SocketNodeHandle ret = (SocketNodeHandle) wr.get();
+      if (ret == null) {
+        return null;
+      }
+      if (ret.getNodeId() == null) {
+        return null;
+      }
+      return ret;
+    }
+  }
+
+  /**
+   * Gets the AddressManager attribute of the SocketSourceRouteManager object
+   *
+   * @param address DESCRIBE THE PARAMETER
+   * @return The AddressManager value
+   */
+  public AddressManager getAddressManager(EpochInetSocketAddress address) {
+    WeakReference wr = (WeakReference) nodeHandles.get(address);
+    if (wr == null) {
+      return null;
     }
 
-    return manager;
+    SocketNodeHandle snh = (SocketNodeHandle) wr.get();
+    if (snh == null) {
+      return null;
+    }
+    return snh.addressManager;
   }
 
   /**
@@ -171,7 +242,7 @@ public class SocketSourceRouteManager {
    */
   protected SourceRoute[] getAllRoutes(EpochInetSocketAddress destination) {
     NodeSet nodes = spn.getLeafSet().neighborSet(Integer.MAX_VALUE);
-    nodes.randomize();
+    nodes.randomize(spn.getEnvironment().getRandomSource());
     Vector result = new Vector();
     result.add(SourceRoute.build(destination));
 
@@ -195,7 +266,7 @@ public class SocketSourceRouteManager {
    * @return The BestRoute value
    */
   protected SourceRoute getBestRoute(EpochInetSocketAddress address) {
-    AddressManager am = (AddressManager) managers.get(address);
+    AddressManager am = getAddressManager(address);
 
     if ((am == null) || (am.getLiveness() == SocketNodeHandle.LIVENESS_DEAD) ||
       (am.getLiveness() == SocketNodeHandle.LIVENESS_DEAD_FOREVER)) {
@@ -211,10 +282,108 @@ public class SocketSourceRouteManager {
    *
    * @exception IOException DESCRIBE THE EXCEPTION
    */
-  public void resign() throws IOException {
-    manager.resign();
+  public void destroy() throws IOException {
+    spn.getEnvironment().getSelectorManager().invoke(
+      new Runnable() {
+        public void run() {
+          try {
+            manager.destroy();
+          } catch (IOException ioe) {
+            if (logger.level <= Logger.WARNING) {
+              logger.logException("Exception while destrying SocketSourceRouteManager", ioe);
+            }
+          }
+        }
+      });
   }
 
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param newHandle DESCRIBE THE PARAMETER
+   * @return DESCRIBE THE RETURN VALUE
+   */
+  public NodeHandle coalesce(NodeHandle newHandle) {
+    SocketNodeHandle snh = (SocketNodeHandle) newHandle;
+    synchronized (nodeHandles) {
+      WeakReference wr = (WeakReference) nodeHandles.get(snh.eaddress);
+      if (wr == null) {
+        addNodeHandle(snh);
+        return snh;
+      } else {
+        SocketNodeHandle ret = (SocketNodeHandle) wr.get();
+        if (ret == null) {
+          // if this happens, then the handle got collected, but not the
+          // eaddress yet. Grumble...
+          addNodeHandle(snh);
+          return snh;
+        } else {
+          // inflates a stub NodeHandle
+          if (ret.getNodeId() == null) {
+            ret.setNodeId(newHandle.getNodeId());
+          }
+          return ret;
+        }
+      }
+    }
+  }
+
+  /**
+   * Adds a feature to the NodeHandle attribute of the SocketSourceRouteManager
+   * object
+   *
+   * @param snh The feature to be added to the NodeHandle attribute
+   */
+  private void addNodeHandle(SocketNodeHandle snh) {
+    WeakReference wr = new WeakReference(snh);
+    nodeHandles.put(snh.eaddress, wr);
+    snh.setLocalNode(spn);
+  }
+
+  /**
+   * Should be called while synchronized on nodeHandles
+   *
+   * @param address
+   * @param search DESCRIBE THE PARAMETER
+   * @return DESCRIBE THE RETURN VALUE
+   */
+  public AddressManager putAddressManager(EpochInetSocketAddress address,
+                                          boolean search) {
+
+    WeakReference wr = (WeakReference) nodeHandles.get(address);
+    SocketNodeHandle snh;
+    AddressManager manager;
+    if (wr == null) {
+      snh = new SocketNodeHandle(address, null);
+      snh.setLocalNode(spn);
+      wr = new WeakReference(snh);
+      nodeHandles.put(address, wr);
+    } else {
+      snh = (SocketNodeHandle) wr.get();
+      if (snh == null) {
+        // WARNING: this code must be repeated because of a very slight timing
+        // issue with the garbage collector
+        snh = new SocketNodeHandle(address, null);
+        snh.setLocalNode(spn);
+        wr = new WeakReference(snh);
+        nodeHandles.put(address, wr);
+      }
+    }
+
+    if (snh.addressManager != null) {
+      throw new IllegalStateException("Address manager for address " + address
+        + " already exists.");
+    }
+
+    manager = new AddressManager(snh, search);
+    // TODO make this time configurable
+    this.spn.getEnvironment().getSelectorManager().getTimer().schedule(
+      new HardLinkTimerTask(manager), 30000);
+    snh.addressManager = manager;
+    return manager;
+  }
+
+
   /**
    * Method which sends a bootstrap message across the wire.
    *
@@ -241,7 +410,7 @@ public class SocketSourceRouteManager {
    * @param address DESCRIBE THE PARAMETER
    */
   public void ping(EpochInetSocketAddress address) {
-    AddressManager am = (AddressManager) managers.get(address);
+    AddressManager am = getAddressManager(address);
 
     if (am == null) {
       manager.ping(SourceRoute.build(address));
@@ -270,7 +439,7 @@ public class SocketSourceRouteManager {
    * @return The ping value to the remote address
    */
   public int proximity(EpochInetSocketAddress address) {
-    AddressManager am = (AddressManager) managers.get(address);
+    AddressManager am = getAddressManager(address);
 
     if (am == null) {
       return SocketNodeHandle.DEFAULT_PROXIMITY;
@@ -285,9 +454,11 @@ public class SocketSourceRouteManager {
    * @param route The now-dead route
    */
   protected void markDead(SourceRoute route) {
-    debug("Found route " + route + " to be dead");
+    if (logger.level <= Logger.FINE) {
+      logger.log("(SSRM) Found route " + route + " to be dead");
+    }
 
-    AddressManager am = (AddressManager) managers.get(route.getLastHop());
+    AddressManager am = getAddressManager(route.getLastHop());
 
     if (am != null) {
       am.markDead(route);
@@ -302,7 +473,7 @@ public class SocketSourceRouteManager {
    * @param address The now-dead address
    */
   protected void markDead(EpochInetSocketAddress address) {
-    AddressManager am = (AddressManager) managers.get(address);
+    AddressManager am = getAddressManager(address);
 
     if (am != null) {
       am.markDeadForever();
@@ -315,7 +486,9 @@ public class SocketSourceRouteManager {
    * @param route The now-live route
    */
   protected void markAlive(SourceRoute route) {
-    debug("Found route " + route + " to be alive");
+    if (logger.level <= Logger.FINE) {
+      logger.log("(SSRM) Found route " + route + " to be alive");
+    }
 
     getAddressManager(route.getLastHop(), false).markAlive(route);
   }
@@ -326,7 +499,9 @@ public class SocketSourceRouteManager {
    * @param route The now-live route
    */
   protected void markSuspected(SourceRoute route) {
-    debug("Found route " + route + " to be suspected");
+    if (logger.level <= Logger.FINE) {
+      logger.log("(SSRM) Found route " + route + " to be suspected");
+    }
 
     getAddressManager(route.getLastHop(), false).markSuspected(route);
   }
@@ -349,32 +524,85 @@ public class SocketSourceRouteManager {
    * @param m The message
    * @param address The address of the remote node
    */
-  protected void reroute(EpochInetSocketAddress address, Message m) {
-    if (getLiveness(address) == SocketNodeHandle.LIVENESS_ALIVE) {
-      debug("Attempting to resend message " + m + " to alive address " + address);
-      send(address, m);
-    } else {
-      if (m instanceof RouteMessage) {
-        if (((RouteMessage) m).getOptions().multipleHopsAllowed()) {
-          debug("Attempting to reroute route message " + m);
-          ((RouteMessage) m).nextHop = null;
-          spn.receiveMessage(m);
-        }
-      } else {
-        System.out.println("Dropping message " + m + " because next hop is dead!");
-      }
-    }
-  }
-
+//  protected void reroutee(EpochInetSocketAddress address, Message m) {
+//    if (getLiveness(address) == SocketNodeHandle.LIVENESS_ALIVE) {
+//      if (logger.level <= Logger.INFO) logger.log( "(SSRM) Attempting to resend message " + m + " to alive address " + address);
+//      send(address, m);
+//    } else {
+//      if (m instanceof RouteMessage) {
+//        if (((RouteMessage) m).getOptions().multipleHopsAllowed()) {
+//          if (logger.level <= Logger.INFO) logger.log( "(SSRM) Attempting to reroute route message " + m);
+//          ((RouteMessage) m).nextHop = null;
+//          // kick it back to pastry
+//          spn.receiveMessage(m);
+//        } else if (getLiveness(address) <= SocketNodeHandle.LIVENESS_SUSPECTED) {
+//          // it's required to go to this address only
+//          send(address, m);
+//        } else {
+//          // this address is dead, and the routemessage is not allowed to go anywhere else
+//          if (logger.level <= Logger.WARNING) logger.log("(SSRM) Dropping message " + m + " because next hop "+address+" is dead!");
+//        }
+//      } else {
+//        if (logger.level <= Logger.WARNING) logger.log("(SSRM) Dropping message " + m + " because next hop "+address+" is dead!");
+//      }
+//    }
+//  }
 
   /**
-   * Debugging method
+   * Reroutes the given message. If this node is alive, send() is called. If
+   * this node is not alive and the message is a route message, it is rerouted.
+   * Otherwise, the message is dropped. Can be called when a socket is closed,
+   * if for example a different source route is found. This is how
+   * non-routemessages may be called here For suspected/dead, it will get called
+   * with all RouteMessages
    *
-   * @param s The string to print
+   * @param m The message
+   * @param address The address of the remote node
    */
-  private void debug(String s) {
-    if (Log.ifp(8)) {
-      System.out.println(spn.getNodeId() + " (SSRM): " + s);
+  protected void reroute(EpochInetSocketAddress address, Message m) {
+
+    switch (getLiveness(address)) {
+      case SocketNodeHandle.LIVENESS_ALIVE:
+        if (logger.level <= Logger.INFO) {
+          logger.log("(SSRM) Attempting to resend message " + m + " to alive address " + address);
+        }
+        send(address, m);
+        return;
+      case SocketNodeHandle.LIVENESS_SUSPECTED:
+        if (m instanceof RouteMessage) {
+          RouteMessage rm = (RouteMessage) m;
+          if (rm.getOptions().multipleHopsAllowed() && rm.getOptions().rerouteIfSuspected()) {
+            // kick it back to pastry
+            if (logger.level <= Logger.INFO) {
+              logger.log("(SSRM) Attempting to reroute route message " + m);
+            }
+            rm.nextHop = null;
+            spn.receiveMessage(rm);
+            return;
+          }
+        } else {
+          if (logger.level <= Logger.INFO) {
+            logger.log("(SSRM) Attempting to resend message " + m + " to alive address " + address);
+          }
+          send(address, m);
+          return;
+        }
+      case SocketNodeHandle.LIVENESS_DEAD:
+      case SocketNodeHandle.LIVENESS_DEAD_FOREVER:
+        if (m instanceof RouteMessage) {
+          RouteMessage rm = (RouteMessage) m;
+          if (rm.getOptions().multipleHopsAllowed()) {
+            if (logger.level <= Logger.INFO) {
+              logger.log("(SSRM) Attempting to reroute route message " + m);
+            }
+            rm.nextHop = null;
+            spn.receiveMessage(rm);
+            return;
+          }
+        }
+    }
+    if (logger.level <= Logger.WARNING) {
+      logger.log("(SSRM) Dropping message " + m + " because next hop " + address + " is dead!");
     }
   }
 
@@ -383,7 +611,7 @@ public class SocketSourceRouteManager {
    * remote address. This class is in charge of all source routes to that
    * address, as well as declaring liveness/death of this address
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected class AddressManager {
@@ -392,11 +620,11 @@ public class SocketSourceRouteManager {
     /**
      * DESCRIBE THE FIELD
      */
-    protected EpochInetSocketAddress address;
+    protected SocketNodeHandle address;
 
-    // the current best route to this remote address
     /**
-     * DESCRIBE THE FIELD
+     * the current best route to this remote address if best == null, we are
+     * already in a CheckDead, which means we are searching for a path
      */
     protected SourceRoute best;
 
@@ -431,20 +659,20 @@ public class SocketSourceRouteManager {
      * @param address The address
      * @param search Whether or not the manager should try and find a route
      */
-    public AddressManager(EpochInetSocketAddress address, boolean search) {
+    public AddressManager(SocketNodeHandle address, boolean search) {
       this.address = address;
       this.queue = new Vector();
       this.routes = new HashMap();
       this.liveness = SocketNodeHandle.LIVENESS_SUSPECTED;
       this.updated = 0L;
 
-      if (SocketPastryNode.verbose) {
-        System.out.println("ADDRESS MANAGER CREATED AT " + localAddress + " FOR " + address);
+      if (logger.level <= Logger.FINE) {
+        logger.log("(SSRM) ADDRESS MANAGER CREATED AT " + localAddress + " FOR " + address);
       }
 
       if (search) {
-        getRouteManager(SourceRoute.build(address)).checkLiveness();
-        this.updated = System.currentTimeMillis();
+        getRouteManager(SourceRoute.build(address.eaddress)).checkLiveness();
+        this.updated = spn.getEnvironment().getTimeSource().currentTimeMillis();
       }
     }
 
@@ -453,8 +681,13 @@ public class SocketSourceRouteManager {
      *
      * @param route The route
      * @return THe manager
+     * @throws IllegalArgumentException if route is null
      */
     protected SourceRouteManager getRouteManager(SourceRoute route) {
+      if (route == null) {
+        throw new IllegalArgumentException("route is null in " + toString());
+      }
+
       SourceRouteManager result = (SourceRouteManager) routes.get(route);
 
       if (result == null) {
@@ -477,30 +710,44 @@ public class SocketSourceRouteManager {
 
     /**
      * Internal method which marks this address as being alive. If we were dead
-     * before, it sends an update out to the observers.
+     * before, it sends an update out to the observers. best must be non-null
+     *
+     * @throws IllegalStateException if best is null.
      */
     protected void setAlive() {
+      if (best == null) {
+        throw new IllegalStateException("best is null in " + toString());
+      }
+
       // we can now send any pending messages
       while (queue.size() > 0) {
         getRouteManager(best).send((Message) queue.remove(0));
       }
 
+      if (queue.isEmpty()) {
+        hardLinks.remove(this);
+      }
+
       switch (liveness) {
         case SocketNodeHandle.LIVENESS_DEAD:
           liveness = SocketNodeHandle.LIVENESS_ALIVE;
-          pool.update(address, SocketNodeHandle.DECLARED_LIVE);
-          if (SocketPastryNode.verbose) {
-            System.out.println("COUNT: " + System.currentTimeMillis() + " " + localAddress + " Found address " + address + " to be alive again.");
+          if (address != null) {
+            address.update(SocketNodeHandle.DECLARED_LIVE);
+          }
+          if (logger.level <= Logger.FINE) {
+            logger.log("COUNT: " + localAddress + " Found address " + address + " to be alive again.");
           }
           break;
         case SocketNodeHandle.LIVENESS_SUSPECTED:
           liveness = SocketNodeHandle.LIVENESS_ALIVE;
-          if (SocketPastryNode.verbose) {
-            System.out.println("COUNT: " + System.currentTimeMillis() + " " + localAddress + " Found address " + address + " to be unsuspected.");
+          if (logger.level <= Logger.FINE) {
+            logger.log("COUNT: " + localAddress + " Found address " + address + " to be unsuspected.");
           }
           break;
         case SocketNodeHandle.LIVENESS_DEAD_FOREVER:
-          System.out.println("ERROR: Found dead-forever handle to " + address + " to be alive again!");
+          if (logger.level <= Logger.WARNING) {
+            logger.log("ERROR: Found dead-forever handle to " + address + " to be alive again!");
+          }
           break;
       }
     }
@@ -512,15 +759,19 @@ public class SocketSourceRouteManager {
       switch (liveness) {
         case SocketNodeHandle.LIVENESS_ALIVE:
           liveness = SocketNodeHandle.LIVENESS_SUSPECTED;
-          if (SocketPastryNode.verbose) {
-            System.out.println("COUNT: " + System.currentTimeMillis() + " " + localAddress + " Found address " + address + " to be suspected.");
+          if (logger.level <= Logger.FINE) {
+            logger.log("COUNT: " + spn.getEnvironment().getTimeSource().currentTimeMillis() + " " + localAddress + " Found address " + address + " to be suspected.");
           }
           break;
         case SocketNodeHandle.LIVENESS_DEAD:
-          System.out.println("ERROR: Found node handle " + address + " to be suspected from dead - should not happen!");
+          if (logger.level <= Logger.WARNING) {
+            logger.log("ERROR: Found node handle " + address + " to be suspected from dead - should not happen!");
+          }
           break;
         case SocketNodeHandle.LIVENESS_DEAD_FOREVER:
-          System.out.println("ERROR: Found node handle " + address + " to be suspected from dead forever - should never ever happen!");
+          if (logger.level <= Logger.WARNING) {
+            logger.log("ERROR: Found node handle " + address + " to be suspected from dead forever - should never ever happen!");
+          }
           break;
       }
 
@@ -529,11 +780,19 @@ public class SocketSourceRouteManager {
 
       for (int i = 0; i < array.length; i++) {
         if (array[i] instanceof RouteMessage) {
-          System.out.println("REROUTE: Rerouting message " + array[i] + " due to suspected next hop " + address);
-          reroute(address, (Message) array[i]);
-          queue.remove(array[i]);
+          RouteMessage rm = (RouteMessage) array[i];
+          if (rm.getOptions().multipleHopsAllowed() && rm.getOptions().rerouteIfSuspected()) {
+            if (logger.level <= Logger.FINE) {
+              logger.log("REROUTE: Rerouting message " + rm + " due to suspected next hop " + address);
+            }
+            reroute(address.eaddress, rm);
+            queue.remove(rm);
+          }
         }
       }
+      if (queue.isEmpty()) {
+        hardLinks.remove(this);
+      }
     }
 
     /**
@@ -545,23 +804,24 @@ public class SocketSourceRouteManager {
         case SocketNodeHandle.LIVENESS_DEAD:
           return;
         case SocketNodeHandle.LIVENESS_DEAD_FOREVER:
-          System.out.println("ERROR: Found node handle " + address + " to be dead from dead forever - should not happen!");
+          if (logger.level <= Logger.WARNING) {
+            logger.log("ERROR: Found node handle " + address + " to be dead from dead forever - should not happen!");
+          }
           break;
         default:
           this.best = null;
           this.liveness = SocketNodeHandle.LIVENESS_DEAD;
-          pool.update(address, SocketNodeHandle.DECLARED_DEAD);
-          manager.declaredDead(address);
-          if (SocketPastryNode.verbose) {
-            System.out.println("COUNT: " + System.currentTimeMillis() + " " + localAddress + " Found address " + address + " to be dead.");
+          if (address != null) {
+            address.update(SocketNodeHandle.DECLARED_DEAD);
+          }
+          manager.declaredDead(address.eaddress);
+          if (logger.level <= Logger.FINE) {
+            logger.log("COUNT: " + localAddress + " Found address " + address + " to be dead.");
           }
           break;
       }
 
-      // and finally we can now send any pending messages
-      while (queue.size() > 0) {
-        reroute(address, (Message) queue.remove(0));
-      }
+      purgeQueue();
     }
 
     /**
@@ -574,24 +834,22 @@ public class SocketSourceRouteManager {
           return;
         case SocketNodeHandle.LIVENESS_DEAD:
           this.liveness = SocketNodeHandle.LIVENESS_DEAD_FOREVER;
-          if (SocketPastryNode.verbose) {
-            System.out.println("COUNT: " + System.currentTimeMillis() + " " + localAddress + " Found address " + address + " to be dead forever.");
+          if (logger.level <= Logger.FINE) {
+            logger.log("COUNT: " + localAddress + " Found address " + address + " to be dead forever.");
           }
           break;
         default:
           this.best = null;
           this.liveness = SocketNodeHandle.LIVENESS_DEAD_FOREVER;
-          pool.update(address, SocketNodeHandle.DECLARED_DEAD);
-          if (SocketPastryNode.verbose) {
-            System.out.println("COUNT: " + System.currentTimeMillis() + " " + localAddress + " Found address " + address + " to be dead forever.");
+          if (address != null) {
+            address.update(SocketNodeHandle.DECLARED_DEAD);
+          }
+          if (logger.level <= Logger.FINE) {
+            logger.log("COUNT: " + localAddress + " Found address " + address + " to be dead forever.");
           }
           break;
       }
-
-      // and finally we can now send any pending messages
-      while (queue.size() > 0) {
-        reroute(address, (Message) queue.remove(0));
-      }
+      purgeQueue();
     }
 
     /**
@@ -618,7 +876,9 @@ public class SocketSourceRouteManager {
 
       // first, we check and see if we have no best route (this can happen if the best just died)
       if (best == null) {
-        debug("No previous best route existed to " + address + " route " + route + " is now the best");
+        if (logger.level <= Logger.FINE) {
+          logger.log("(SSRM) No previous best route existed to " + address + " route " + route + " is now the best");
+        }
         best = route;
       }
 
@@ -627,10 +887,15 @@ public class SocketSourceRouteManager {
       if ((best.getNumHops() > route.getNumHops()) ||
         ((best.getNumHops() == route.getNumHops()) &&
         (getRouteManager(best).proximity() > getRouteManager(route).proximity()))) {
-        debug("Route " + route + " is better than previous best route " + best + " - replacing");
+        if (logger.level <= Logger.FINE) {
+          logger.log("(SSRM) Route " + route + " is better than previous best route " + best + " - replacing");
+        }
 
         best = route;
-        pool.update(address, SocketNodeHandle.PROXIMITY_CHANGED);
+
+        if (address != null) {
+          address.update(SocketNodeHandle.PROXIMITY_CHANGED);
+        }
       }
 
       // finally, mark this address as alive
@@ -705,17 +970,22 @@ public class SocketSourceRouteManager {
     protected synchronized void markProximity(SourceRoute route, int proximity) {
       getRouteManager(route).markAlive();
       getRouteManager(route).markProximity(proximity);
-      setAlive();
 
       // first, we check and see if we have no best route (this can happen if the best just died)
       if (best == null) {
-        debug("No previous best route existed to " + address + " route " + route + " is now the best");
+        if (logger.level <= Logger.FINE) {
+          logger.log("(SSRM) No previous best route existed to " + address + " route " + route + " is now the best");
+        }
         best = route;
       }
 
+      setAlive();
+
       // next, we update everyone if this is the active route
       if (route.equals(best)) {
-        pool.update(address, SocketNodeHandle.PROXIMITY_CHANGED);
+        if (address != null) {
+          address.update(SocketNodeHandle.PROXIMITY_CHANGED);
+        }
       }
     }
 
@@ -727,20 +997,22 @@ public class SocketSourceRouteManager {
     public synchronized void send(Message message) {
       // if we're dead, we go ahead and just checkDead on the direct route
       if (liveness == SocketNodeHandle.LIVENESS_DEAD) {
-        getRouteManager(SourceRoute.build(address)).checkLiveness();
-        this.updated = System.currentTimeMillis();
+        getRouteManager(SourceRoute.build(address.eaddress)).checkLiveness();
+        this.updated = spn.getEnvironment().getTimeSource().currentTimeMillis();
       }
 
       // and in any case, we either send if we have a best route or add the message
       // to the queue
       if (best == null) {
         queue.add(message);
+        hardLinks.add(this);
       } else if (!getRouteManager(best).isOpen()) {
         queue.add(message);
+        hardLinks.add(this);
 
         getRouteManager(best).checkLiveness();
         this.best = null;
-        this.updated = System.currentTimeMillis();
+        this.updated = spn.getEnvironment().getTimeSource().currentTimeMillis();
       } else {
         getRouteManager(best).send(message);
       }
@@ -750,15 +1022,17 @@ public class SocketSourceRouteManager {
      * Method which suggests a ping to the remote node.
      */
     public void ping() {
-      if (System.currentTimeMillis() - updated > PING_THROTTLE) {
-        this.updated = System.currentTimeMillis();
+      if (spn.getEnvironment().getTimeSource().currentTimeMillis() - updated > PING_THROTTLE) {
+        this.updated = spn.getEnvironment().getTimeSource().currentTimeMillis();
 
         switch (liveness) {
           case SocketNodeHandle.LIVENESS_DEAD_FOREVER:
             return;
           case SocketNodeHandle.LIVENESS_DEAD:
-            System.out.println("PING: PINGING DEAD ADDRESS " + address + " - JUST IN CASE, NO HARM ANYWAY");
-            getRouteManager(SourceRoute.build(address)).ping();
+            if (logger.level <= Logger.FINE) {
+              logger.log("(SSRM) PING: PINGING DEAD ADDRESS " + address + " - JUST IN CASE, NO HARM ANYWAY");
+            }
+            getRouteManager(SourceRoute.build(address.eaddress)).ping();
             break;
           default:
             if (best != null) {
@@ -766,7 +1040,7 @@ public class SocketSourceRouteManager {
 
               // check to see if the direct route is available
               if (!best.isDirect()) {
-                getRouteManager(SourceRoute.build(address)).ping();
+                getRouteManager(SourceRoute.build(address.eaddress)).ping();
               }
             }
 
@@ -779,14 +1053,16 @@ public class SocketSourceRouteManager {
      * Method which suggests a ping to the remote node.
      */
     public void checkLiveness() {
-      this.updated = System.currentTimeMillis();
+      this.updated = spn.getEnvironment().getTimeSource().currentTimeMillis();
 
       switch (liveness) {
         case SocketNodeHandle.LIVENESS_DEAD_FOREVER:
           return;
         case SocketNodeHandle.LIVENESS_DEAD:
-          System.out.println("CHECKLIVENESS: CHECKING DEAD ON DEAD ADDRESS " + address + " - JUST IN CASE, NO HARM ANYWAY");
-          getRouteManager(SourceRoute.build(address)).checkLiveness();
+          if (logger.level <= Logger.FINE) {
+            logger.log("(SSRM) CHECKLIVENESS: CHECKING DEAD ON DEAD ADDRESS " + address + " - JUST IN CASE, NO HARM ANYWAY");
+          }
+          getRouteManager(SourceRoute.build(address.eaddress)).checkLiveness();
           break;
         default:
           if (best != null) {
@@ -794,7 +1070,7 @@ public class SocketSourceRouteManager {
 
             // check to see if the direct route is available
             if (!best.isDirect()) {
-              getRouteManager(SourceRoute.build(address)).checkLiveness();
+              getRouteManager(SourceRoute.build(address.eaddress)).checkLiveness();
             }
           }
 
@@ -802,11 +1078,22 @@ public class SocketSourceRouteManager {
       }
     }
 
+    /**
+     * DESCRIBE THE METHOD
+     */
+    protected void purgeQueue() {
+      // and finally we can now send any pending messages
+      while (queue.size() > 0) {
+        reroute(address.eaddress, (Message) queue.remove(0));
+      }
+      hardLinks.remove(this);
+    }
+
     /**
      * Internal class which is charges with managing the remote connection via a
      * specific route
      *
-     * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+     * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
      * @author jeffh
      */
     public class SourceRouteManager {
@@ -847,6 +1134,9 @@ public class SocketSourceRouteManager {
        * @param route The route
        */
       public SourceRouteManager(SourceRoute route) {
+        if (route == null) {
+          throw new IllegalArgumentException("route is null");
+        }
         this.route = route;
         this.liveness = SocketNodeHandle.LIVENESS_SUSPECTED;
         this.proximity = SocketNodeHandle.DEFAULT_PROXIMITY;
@@ -921,8 +1211,8 @@ public class SocketSourceRouteManager {
         }
 
         if ((this.liveness < SocketNodeHandle.LIVENESS_DEAD) ||
-          (this.updated < System.currentTimeMillis() - CHECK_DEAD_THROTTLE)) {
-          this.updated = System.currentTimeMillis();
+          (this.updated < spn.getEnvironment().getTimeSource().currentTimeMillis() - CHECK_DEAD_THROTTLE)) {
+          this.updated = spn.getEnvironment().getTimeSource().currentTimeMillis();
           this.pending = true;
           manager.checkLiveness(route);
           return true;
@@ -937,7 +1227,7 @@ public class SocketSourceRouteManager {
        * @param message The message to send
        */
       public synchronized void send(Message message) {
-        manager.send(route, message);
+        manager.send(route, message, AddressManager.this);
       }
 
       /**
@@ -948,4 +1238,30 @@ public class SocketSourceRouteManager {
       }
     }
   }
+
+  /**
+   * DESCRIBE THE CLASS
+   *
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
+   * @author jeffh
+   */
+  static class HardLinkTimerTask extends TimerTask {
+    AddressManager manager;
+
+    /**
+     * Constructor for HardLinkTimerTask.
+     *
+     * @param manager DESCRIBE THE PARAMETER
+     */
+    public HardLinkTimerTask(AddressManager manager) {
+      this.manager = manager;
+    }
+
+    /**
+     * Main processing method for the HardLinkTimerTask object
+     */
+    public void run() {
+      // do nothing, just expire
+    }
+  }
 }
