@@ -33,29 +33,29 @@ or otherwise) arising in any way out of the use of this software, even
 if advised of the possibility of such damage.
 
 ********************************************************************************/
-
 package rice.selector;
 
 import java.io.IOException;
-import java.nio.channels.SelectableChannel;
-import java.nio.channels.SelectionKey;
-import java.nio.channels.Selector;
+import java.nio.channels.*;
 import java.util.ArrayList;
-import java.util.Date;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.TreeSet;
 
+import rice.Destructable;
+import rice.environment.logging.*;
+import rice.environment.time.TimeSource;
+
 /**
  * This class is the class which handles the selector, and listens for activity.
  * When activity occurs, it figures out who is interested in what has happened,
  * and hands off to that object.
  *
- * @version $Id: SelectorManager.java,v 1.27 2005/05/17 15:42:18 jeffh Exp $
+ * @version $Id: SelectorManager.java 2870 2005-12-21 15:07:14Z jeffh $
  * @author Alan Mislove
  */
-public class SelectorManager extends Thread implements Timer {
+public class SelectorManager extends Thread implements Timer, Destructable {
 
   // the underlying selector used
   /**
@@ -93,7 +93,28 @@ public class SelectorManager extends Thread implements Timer {
    */
   protected long wakeupTime = 0;
 
-  long lastTime = System.currentTimeMillis();
+  /**
+   * DESCRIBE THE FIELD
+   */
+  protected TimeSource timeSource;
+
+  long lastTime = 0;
+
+//  protected LogManager log;
+  /**
+   * DESCRIBE THE FIELD
+   */
+  protected Logger logger;
+
+  /**
+   * DESCRIBE THE FIELD
+   */
+  protected String instance;
+
+  /**
+   * DESCRIBE THE FIELD
+   */
+  protected boolean running = true;
 
   ArrayList loopObservers = new ArrayList();
 
@@ -103,27 +124,32 @@ public class SelectorManager extends Thread implements Timer {
    */
   public static int TIMEOUT = 500;
 
-  // The static selector manager which is used by all applications
-  private static SelectorManager manager;
-
   /**
    * Constructor, which is private since there is only one selector per JVM.
    *
-   * @param profile DESCRIBE THE PARAMETER
+   * @param instance DESCRIBE THE PARAMETER
+   * @param timeSource DESCRIBE THE PARAMETER
+   * @param log DESCRIBE THE PARAMETER
    */
-  protected SelectorManager(boolean profile) {
-    super("Main Selector Thread");
+  public SelectorManager(String instance,
+                         TimeSource timeSource, LogManager log) {
+    super(instance == null ? "Selector Thread" : "Selector Thread -- "
+      + instance);
+    this.instance = instance;
+    this.logger = log.getLogger(getClass(), instance);
     this.invocations = new LinkedList();
     this.modifyKeys = new HashSet();
     this.cancelledKeys = new HashSet();
+    this.timeSource = timeSource;
 
     // attempt to create selector
     try {
       selector = Selector.open();
     } catch (IOException e) {
-      System.out.println("SEVERE ERROR (SelectorManager): Error creating selector " + e);
+      System.out.println("SEVERE ERROR (SelectorManager): Error creating selector "
+        + e);
     }
-
+    lastTime = timeSource.currentTimeMillis();
     start();
   }
 
@@ -176,6 +202,15 @@ public class SelectorManager extends Thread implements Timer {
     }
   }
 
+  /**
+   * Returns whether or not this thread of execution is the selector thread
+   *
+   * @return Whether or not this is the selector thread
+   */
+  public boolean isSelectorThread() {
+    return (Thread.currentThread() == this);
+  }
+
   /**
    * Returns the timer associated with this SelectorManager (in this case, it is
    * this).
@@ -186,6 +221,15 @@ public class SelectorManager extends Thread implements Timer {
     return this;
   }
 
+  /**
+   * Gets the Selector attribute of the SelectorManager object
+   *
+   * @return The Selector value
+   */
+  public Selector getSelector() {
+    return selector;
+  }
+
   /**
    * Method which asks the Selector Manager to add the given key to the
    * cancelled set. If noone calls register on this key during the rest of this
@@ -213,7 +257,8 @@ public class SelectorManager extends Thread implements Timer {
    * @return The SelectionKey which uniquely identifies this channel
    * @exception IOException DESCRIBE THE EXCEPTION
    */
-  public SelectionKey register(SelectableChannel channel, SelectionKeyHandler handler, int ops) throws IOException {
+  public SelectionKey register(SelectableChannel channel,
+                               SelectionKeyHandler handler, int ops) throws IOException {
     if ((channel == null) || (handler == null)) {
       throw new NullPointerException();
     }
@@ -270,14 +315,17 @@ public class SelectorManager extends Thread implements Timer {
   public void run() {
     try {
       //System.out.println("SelectorManager starting...");
-      debug("SelectorManager starting...");
+      if (logger.level <= Logger.INFO) {
+        logger.log("SelectorManager -- " + instance + " starting...");
+      }
 
-      lastTime = System.currentTimeMillis();
+      lastTime = timeSource.currentTimeMillis();
       // loop while waiting for activity
-      while (true) {
+      while (running) {
         notifyLoopListeners();
 
-        // NOTE: This is so we aren't always holding the selector lock when we get context switched
+        // NOTE: This is so we aren't always holding the selector lock when we
+        // get context switched
         Thread.yield();
         executeDueTasks();
         onLoop();
@@ -287,7 +335,7 @@ public class SelectorManager extends Thread implements Timer {
           int selectTime = SelectorManager.TIMEOUT;
           if (timerQueue.size() > 0) {
             TimerTask first = (TimerTask) timerQueue.first();
-            selectTime = (int) (first.nextExecutionTime - System.currentTimeMillis());
+            selectTime = (int) (first.nextExecutionTime - timeSource.currentTimeMillis());
           }
 
           select(selectTime);
@@ -301,23 +349,37 @@ public class SelectorManager extends Thread implements Timer {
 
             cancelledKeys.clear();
 
-            // now, hack to make sure that all cancelled keys are actually cancelled (dumb)
+            // now, hack to make sure that all cancelled keys are actually
+            // cancelled (dumb)
             selector.selectNow();
           }
         }
       }
     } catch (Throwable t) {
-      System.out.println("ERROR (SelectorManager.run): " + t);
-      t.printStackTrace(System.out);
+      if (logger.level <= Logger.SEVERE) {
+        logger.logException(
+          "ERROR (SelectorManager.run): ", t);
+      }
       System.exit(-1);
     }
+    if (logger.level <= Logger.INFO) {
+      logger.log("Selector " + instance + " shutting down.");
+    }
+  }
+
+
+  /**
+   * DESCRIBE THE METHOD
+   */
+  public void destroy() {
+    running = false;
   }
 
   /**
    * DESCRIBE THE METHOD
    */
   protected void notifyLoopListeners() {
-    long now = System.currentTimeMillis();
+    long now = timeSource.currentTimeMillis();
     long diff = now - lastTime;
     // notify observers
     synchronized (loopObservers) {
@@ -354,7 +416,6 @@ public class SelectorManager extends Thread implements Timer {
     }
   }
 
-
   /**
    * DESCRIBE THE METHOD
    *
@@ -413,8 +474,10 @@ public class SelectorManager extends Thread implements Timer {
       try {
         run.run();
       } catch (Exception e) {
-        System.err.println("Invoking runnable caused exception " + e + " - continuing");
-        e.printStackTrace();
+        if (logger.level <= Logger.SEVERE) {
+          logger.logException(
+            "Invoking runnable caused exception " + e + " - continuing", e);
+        }
       }
     }
 
@@ -449,11 +512,18 @@ public class SelectorManager extends Thread implements Timer {
         return selector.selectNow();
       }
 
-      wakeupTime = System.currentTimeMillis() + time;
+      wakeupTime = timeSource.currentTimeMillis() + time;
       return selector.select(time);
+    } catch (CancelledKeyException cce) {
+      if (logger.level <= Logger.WARNING) {
+        logger.logException("CCE: cause:", cce.getCause());
+      }
+      throw cce;
     } catch (IOException e) {
       if (e.getMessage().indexOf("Interrupted system call") >= 0) {
-        System.out.println("Got interrupted system call, continuing anyway...");
+        if (logger.level <= Logger.WARNING) {
+          logger.log("Got interrupted system call, continuing anyway...");
+        }
         return 1;
       } else {
         throw e;
@@ -480,39 +550,18 @@ public class SelectorManager extends Thread implements Timer {
    * @exception IOException DESCRIBE THE EXCEPTION
    */
   protected SelectionKey[] selectedKeys() throws IOException {
-    return (SelectionKey[]) selector.selectedKeys().toArray(new SelectionKey[0]);
-  }
-
-  /**
-   * DESCRIBE THE METHOD
-   *
-   * @param s DESCRIBE THE PARAMETER
-   */
-  private void debug(String s) {
-    if (rice.pastry.Log.ifp(8)) {
-      System.out.println("(SelectorManager): " + s);
-    }
+    return (SelectionKey[]) selector.selectedKeys()
+      .toArray(new SelectionKey[0]);
   }
 
   /**
    * Method which schedules a task to run after a specified number of millis
    *
    * @param task The task to run
-   * @param delay The delay
+   * @param delay The delay before running, in milliseconds
    */
   public void schedule(TimerTask task, long delay) {
-    task.nextExecutionTime = System.currentTimeMillis() + delay;
-    addTask(task);
-  }
-
-  /**
-   * Method which schedules a task to run at a specified time
-   *
-   * @param task The task to run
-   * @param time The time to run
-   */
-  public void schedule(TimerTask task, Date time) {
-    task.nextExecutionTime = time.getTime();
+    task.nextExecutionTime = timeSource.currentTimeMillis() + delay;
     addTask(task);
   }
 
@@ -521,25 +570,11 @@ public class SelectorManager extends Thread implements Timer {
    * period
    *
    * @param task The task to run
-   * @param delay The delay
-   * @param period The period with which to run
+   * @param delay The delay before first running, in milliseconds
+   * @param period The period with which to run in milliseconds
    */
   public void schedule(TimerTask task, long delay, long period) {
-    task.nextExecutionTime = System.currentTimeMillis() + delay;
-    task.period = (int) period;
-    addTask(task);
-  }
-
-  /**
-   * Method which schedules a task to run repeatedly first at a specified time
-   * and period
-   *
-   * @param task The task to run
-   * @param firstTime The first time
-   * @param period The period with which to run
-   */
-  public void schedule(TimerTask task, Date firstTime, long period) {
-    task.nextExecutionTime = firstTime.getTime();
+    task.nextExecutionTime = timeSource.currentTimeMillis() + delay;
     task.period = (int) period;
     addTask(task);
   }
@@ -549,26 +584,13 @@ public class SelectorManager extends Thread implements Timer {
    * specified delay and period
    *
    * @param task The task to run
-   * @param delay The delay
-   * @param period The period with which to run
+   * @param delay The delay before first running in milliseconds
+   * @param period The period with which to run in milliseconds
    */
   public void scheduleAtFixedRate(TimerTask task, long delay, long period) {
-    task.nextExecutionTime = System.currentTimeMillis() + delay;
-    task.period = (int) period;
-    addTask(task);
-  }
-
-  /**
-   * Method which schedules a task to run repeatedly (at a fixed rate) after a
-   * specified delay and period
-   *
-   * @param task The task to run
-   * @param period The period with which to run
-   * @param firstTime DESCRIBE THE PARAMETER
-   */
-  public void scheduleAtFixedRate(TimerTask task, Date firstTime, long period) {
-    task.nextExecutionTime = firstTime.getTime();
+    task.nextExecutionTime = timeSource.currentTimeMillis() + delay;
     task.period = (int) period;
+    task.fixedRate = true;
     addTask(task);
   }
 
@@ -581,7 +603,8 @@ public class SelectorManager extends Thread implements Timer {
   private void addTask(TimerTask task) {
     synchronized (selector) {
       if (!timerQueue.add(task)) {
-        System.out.println("ERROR: Got false while enqueueing task " + task + "!");
+        System.out.println("ERROR: Got false while enqueueing task " + task
+          + "!");
         Thread.dumpStack();
       }
     }
@@ -597,7 +620,7 @@ public class SelectorManager extends Thread implements Timer {
    */
   protected void executeDueTasks() {
     //System.out.println("SM.executeDueTasks()");
-    long now = System.currentTimeMillis();
+    long now = timeSource.currentTimeMillis();
     ArrayList executeNow = new ArrayList();
 
     // step 1, fetch all due timers
@@ -627,11 +650,13 @@ public class SelectorManager extends Thread implements Timer {
       TimerTask next = (TimerTask) i.next();
       try {
         //System.out.println("SM.Executing "+next);
-        if (next.execute()) {
+        if (next.execute(timeSource)) {
           addBack.add(next);
         }
       } catch (Exception e) {
-        e.printStackTrace();
+        if (logger.level <= Logger.SEVERE) {
+          logger.logException("", e);
+        }
       }
     }
 
@@ -645,39 +670,4 @@ public class SelectorManager extends Thread implements Timer {
       }
     }
   }
-
-  /**
-   * Returns the SelectorManager applications should use.
-   *
-   * @return The SelectorManager which applications should use
-   */
-  public static SelectorManager getSelectorManager() {
-    if (manager != null) {
-      return manager;
-    }
-
-    synchronized (SelectorManager.class) {
-      if (manager != null) {
-        return manager;
-      }
-
-      String s = System.getProperty("PROFILE_SELECTOR_MANAGER");
-      if ((s != null) && (s != "")) {
-        System.out.println("Using Profile Selector");
-        manager = new ProfileSelector();
-      } else {
-        manager = new SelectorManager(false);
-      }
-      return manager;
-    }
-  }
-
-  /**
-   * Returns whether or not this thread of execution is the selector thread
-   *
-   * @return Whether or not this is the selector thread
-   */
-  public static boolean isSelectorThread() {
-    return (Thread.currentThread() == manager);
-  }
 }
